@@ -38,6 +38,28 @@
 #include "types.h"
 #include "blowfish.h"
 
+#define CIPHER_ALGO_BLOWFISH	 4  /* blowfish 128 bit key */
+#define CIPHER_ALGO_BLOWFISH160 42  /* blowfish 160 bit key (not in OpenPGP)*/
+
+#define FNCCAST_SETKEY(f)  (void(*)(void*, byte*, unsigned))(f)
+#define FNCCAST_CRYPT(f)   (void(*)(void*, byte*, byte*))(f)
+
+#define BLOWFISH_BLOCKSIZE 8
+#define BLOWFISH_ROUNDS 16
+
+typedef struct {
+    u32 s0[256];
+    u32 s1[256];
+    u32 s2[256];
+    u32 s3[256];
+    u32 p[BLOWFISH_ROUNDS+2];
+} BLOWFISH_context;
+
+static void bf_setkey( BLOWFISH_context *c, byte *key, unsigned keylen );
+static void encrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf );
+static void decrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf );
+
+
 /* precomputed S boxes */
 static const u32 ks0[256] = {
     0xD1310BA6,0x98DFB5AC,0x2FFD72DB,0xD01ADFB7,0xB8E1AFED,0x6A267E96,
@@ -391,8 +413,8 @@ decrypt(  BLOWFISH_context *bc, u32 *ret_xl, u32 *ret_xr )
 #undef F
 #undef R
 
-void
-blowfish_encrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
+static void
+encrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
 {
     u32 d1, d2;
 
@@ -410,8 +432,8 @@ blowfish_encrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
 }
 
 
-void
-blowfish_decrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
+static void
+decrypt_block( BLOWFISH_context *bc, byte *outbuf, byte *inbuf )
 {
     u32 d1, d2;
 
@@ -439,27 +461,27 @@ selftest()
     byte key3[] = { 0x41, 0x79, 0x6E, 0xA0, 0x52, 0x61, 0x6E, 0xE4 };
     byte cipher3[] = { 0xE1, 0x13, 0xF4, 0x10, 0x2C, 0xFC, 0xCE, 0x43 };
 
-    blowfish_setkey( &c, "abcdefghijklmnopqrstuvwxyz", 26 );
-    blowfish_encrypt_block( &c, buffer, plain );
+    bf_setkey( &c, "abcdefghijklmnopqrstuvwxyz", 26 );
+    encrypt_block( &c, buffer, plain );
     if( memcmp( buffer, "\x32\x4E\xD0\xFE\xF4\x13\xA2\x03", 8 ) )
 	log_error("wrong blowfish encryption\n");
-    blowfish_decrypt_block( &c, buffer, buffer );
+    decrypt_block( &c, buffer, buffer );
     if( memcmp( buffer, plain, 8 ) )
 	log_bug("blowfish failed\n");
 
-    blowfish_setkey( &c, key3, 8 );
-    blowfish_encrypt_block( &c, buffer, plain3 );
+    bf_setkey( &c, key3, 8 );
+    encrypt_block( &c, buffer, plain3 );
     if( memcmp( buffer, cipher3, 8 ) )
 	log_error("wrong blowfish encryption (3)\n");
-    blowfish_decrypt_block( &c, buffer, buffer );
+    decrypt_block( &c, buffer, buffer );
     if( memcmp( buffer, plain3, 8 ) )
 	log_bug("blowfish failed (3)\n");
 }
 
 
 
-void
-blowfish_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
+static void
+bf_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
 {
     int i, j;
     u32 data, datal, datar;
@@ -524,3 +546,31 @@ blowfish_setkey( BLOWFISH_context *c, byte *key, unsigned keylen )
 }
 
 
+/****************
+ * Return some information about the algorithm.  We need algo here to
+ * distinguish different flavors of the algorithm.
+ * Returns: A pointer to string describing the algorithm or NULL if
+ *	    the ALGO is invalid.
+ */
+const char *
+blowfish_get_info( int algo, size_t *keylen,
+		   size_t *blocksize, size_t *contextsize,
+		   void (**r_setkey)( void *c, byte *key, unsigned keylen ),
+		   void (**r_encrypt)( void *c, byte *outbuf, byte *inbuf ),
+		   void (**r_decrypt)( void *c, byte *outbuf, byte *inbuf )
+		 )
+{
+    *keylen = algo == CIPHER_ALGO_BLOWFISH ? 128 : 160;
+    *blocksize = BLOWFISH_BLOCKSIZE;
+    *contextsize = sizeof(BLOWFISH_context);
+    *r_setkey = FNCCAST_SETKEY(bf_setkey);
+    *r_encrypt= FNCCAST_CRYPT(encrypt_block);
+    *r_decrypt= FNCCAST_CRYPT(decrypt_block);
+
+    if( algo == CIPHER_ALGO_BLOWFISH )
+	return "BLOWFISH";
+    if( algo == CIPHER_ALGO_BLOWFISH160 )
+	return "BLOWFISH160";
+    return NULL;
+}
+
