@@ -32,7 +32,7 @@ void add_randomness( const void *buffer, size_t length, int source );
 
 
 /*-- the next two functions are implemented by all the system
-     specific source files rand_xxxx.s --*/
+     specific source files rand-xxxx.c --*/
 void random_poll(void);
 void fast_random_poll(void);
 
