@@ -111,6 +111,8 @@ static ARGPARSE_OPTS opts[] = {
     { 534, "no-comment", 0,   N_("do not write comment packets")},
     { 535, "completes-needed", 1, N_("(default is 1)")},
     { 536, "marginals-needed", 1, N_("(default is 3)")},
+    { 560, "load-extension" ,2, N_("|file|load extension module")},
+    { 561, "rfc1991",   0, N_("emulate the mode described in RFC1991")},
   #ifdef IS_G10
     { 527, "cipher-algo", 2 , N_("|NAME|use cipher algorithm NAME")},
     { 528, "pubkey-algo", 2 , N_("|NAME|use public key algorithm NAME")},
@@ -162,6 +164,7 @@ static ARGPARSE_OPTS opts[] = {
     { 553, "skip-verify",0, "@" },
     { 557, "compress-keys",0, "@"},
     { 559, "always-trust", 0, "@"},
+    { 562, "emulate-checksum-bug", 0, "@"},
 
 {0} };
 
@@ -315,9 +318,10 @@ set_debug(void)
     if( opt.debug & DBG_MPI_VALUE )
 	mpi_debug_mode = 1;
     if( opt.debug & DBG_CIPHER_VALUE )
-	cipher_debug_mode = 1;
+	g10c_debug_mode = 1;
     if( opt.debug & DBG_IOBUF_VALUE )
 	iobuf_debug_mode = 1;
+
 }
 
 
@@ -427,7 +431,7 @@ main( int argc, char **argv )
     orig_argv = argv;
     pargs.argc = &argc;
     pargs.argv = &argv;
-    pargs.flags=  1;  /* do not remove the args */
+    pargs.flags= 1|(1<<6);  /* do not remove the args, ignore version */
     while( arg_parse( &pargs, opts) ) {
 	if( pargs.r_opt == 510 || pargs.r_opt == 511 )
 	    parse_debug++;
@@ -540,7 +544,8 @@ main( int argc, char **argv )
 	#endif /* IS_G10MAINT */
 
 	  case 'o': opt.outfile = pargs.r.ret_str; break;
-	  case 'v': opt.verbose++; opt.list_sigs=1; break;
+	  case 'v': g10_opt_verbose++;
+		    opt.verbose++; opt.list_sigs=1; break;
 	  case 'k': set_cmd( &cmd, aKMode ); break;
 
 	  case 500: opt.batch = 1; greeting = 0; break;
@@ -565,7 +570,8 @@ main( int argc, char **argv )
 	  case 520: default_keyring = 0; break;
 	  case 521: set_cmd( &cmd, aListPackets); break;
 	  case 522: greeting = 0; break;
-	  case 526: opt.verbose = 0; opt.list_sigs=0; break;
+	  case 526: g10_opt_verbose = 0;
+		    opt.verbose = 0; opt.list_sigs=0; break;
 	  case 530: set_cmd( &cmd, aImport); break;
 	  case 532: quick_random_gen(1); break;
 	  case 534: opt.no_comment=1; break;
@@ -586,6 +592,9 @@ main( int argc, char **argv )
 	  case 557: opt.compress_keys = 1; break;
 	  case 558: set_cmd( &cmd, aListSecretKeys); break;
 	  case 559: opt.always_trust = 1; break;
+	  case 560: register_cipher_extension(pargs.r.ret_str); break;
+	  case 561: opt.rfc1991 = 1; break;
+	  case 562: opt.emulate_bugs |= 1; break;
 	  default : errors++; pargs.err = configfp? 1:2; break;
 	}
     }
@@ -629,6 +638,7 @@ main( int argc, char **argv )
 	    opt.list_sigs++;
 
 	opt.verbose = opt.verbose > 1;
+	g10_opt_verbose = opt.verbose;
     }
 
 
@@ -1098,9 +1108,8 @@ print_mds( const char *fname, int algo )
 	md_enable( md, DIGEST_ALGO_MD5 );
 	md_enable( md, DIGEST_ALGO_SHA1 );
 	md_enable( md, DIGEST_ALGO_RMD160 );
-      #ifdef WITH_TIGER_HASH
-	md_enable( md, DIGEST_ALGO_TIGER );
-      #endif
+	if( !check_digest_algo(DIGEST_ALGO_TIGER) )
+	    md_enable( md, DIGEST_ALGO_TIGER );
     }
 
     while( (n=fread( buf, 1, DIM(buf), fp )) )
@@ -1121,10 +1130,10 @@ print_mds( const char *fname, int algo )
 			    print_hex(md_read(md, DIGEST_ALGO_SHA1), 20 );
 	    printf("\n%sRMD160 = ", fname?pname:""  );
 			    print_hex(md_read(md, DIGEST_ALGO_RMD160), 20 );
-	  #ifdef WITH_TIGER_HASH
-	    printf("\n%s TIGER = ", fname?pname:""  );
+	    if( !check_digest_algo(DIGEST_ALGO_TIGER) ) {
+		printf("\n%s TIGER = ", fname?pname:""  );
 			    print_hex(md_read(md, DIGEST_ALGO_TIGER), 24 );
-	  #endif
+	    }
 	}
 	putchar('\n');
     }
