@@ -47,6 +47,8 @@ struct {
     int skip_verify;
     int compress_keys;
     int always_trust;
+    int rfc1991;
+    unsigned emulate_bugs; /* bug emulation flags */
 } opt;
 
 
