@@ -27,12 +27,6 @@
 #include "cipher.h"
 #include "filter.h"
 
-#ifndef HAVE_RSA_CIPHER
-/* although we don't have RSA we need these structures to handle keyrings */
-typedef struct { MPI e, n;	       } RSA_public_key;
-typedef struct { MPI e, n, p, q, d, u; } RSA_secret_key;
-#endif
-
 typedef enum {
 	PKT_NONE	   =0,
 	PKT_PUBKEY_ENC	   =1, /* public key encrypted packet */
@@ -74,14 +68,7 @@ typedef struct {
     u32     keyid[2];	    /* 64 bit keyid */
     byte    version;
     byte    pubkey_algo;    /* algorithm used for public key scheme */
-    union {
-      struct {
-	MPI  a, b;	    /* integers with the encrypteded DEK */
-      } elg;
-      struct {
-	MPI  rsa_integer;   /* integer containing the DEK */
-      } rsa;
-    } d;
+    MPI     data[PUBKEY_MAX_NENC];
 } PKT_pubkey_enc;
 
 
@@ -106,20 +93,16 @@ typedef struct {
     byte *hashed_data;	    /* all subpackets with hashed  data (v4 only) */
     byte *unhashed_data;    /* ditto for unhashed data */
     byte digest_start[2];   /* first 2 bytes of the digest */
-    union {
-      struct {
-	MPI  a, b;	      /* integers with the digest */
-      } elg;
-      struct {
-	MPI  r, s;	      /* integers with the digest */
-      } dsa;
-      struct {
-	MPI  rsa_integer;     /* the encrypted digest */
-      } rsa;
-    } d;
+    MPI  data[PUBKEY_MAX_NSIG];
 } PKT_signature;
 
 
+/****************
+ * Note about the pkey/skey elements:  We assume that the secret keys
+ * has the same elemts as the public key at the begin of the array, so
+ * that npkey < nskey and it is possible to compare the secret and
+ * public keys by comparing the first npkey elements of pkey againts skey.
+ */
 typedef struct {
     u32     timestamp;	    /* certificate made */
     u16     valid_days;     /* valid for this number of days */
@@ -127,11 +110,7 @@ typedef struct {
     byte    version;
     byte    pubkey_algo;    /* algorithm used for public key scheme */
     ulong   local_id;	    /* internal use, valid if > 0 */
-    union {
-	ELG_public_key elg;
-	DSA_public_key dsa;
-	RSA_public_key rsa;
-    } d;
+    MPI     pkey[PUBKEY_MAX_NPKEY];
 } PKT_public_cert;
 
 typedef struct {
@@ -149,11 +128,7 @@ typedef struct {
 	STRING2KEY s2k;
 	byte iv[8]; /* initialization vector for CFB mode */
     } protect;
-    union {
-	ELG_secret_key elg;
-	DSA_secret_key dsa;
-	RSA_secret_key rsa;
-    } d;
+    MPI skey[PUBKEY_MAX_NSKEY];
     u16 csum;		/* checksum */
 } PKT_secret_cert;
 
@@ -234,7 +209,8 @@ typedef enum {
     SIGSUBPKT_PRIMARY_UID  =25, /* primary user id */
     SIGSUBPKT_POLICY	   =26, /* policy URL */
     SIGSUBPKT_KEY_FLAGS    =27, /* key flags */
-    SIGSUBPKT_SIGNERS_UID  =28	/* signer's user id */
+    SIGSUBPKT_SIGNERS_UID  =28, /* signer's user id */
+    SIGSUBPKT_PRIV_ADD_SIG =101 /* signatur is also valid for this uid */
 } sigsubpkttype_t;
 
 
@@ -300,7 +276,7 @@ int decrypt_data( PKT_encrypted *ed, DEK *dek );
 int encrypt_data( PKT_encrypted *ed, DEK *dek );
 
 /*-- plaintext.c --*/
-int handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx );
+int handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,int nooutput);
 int ask_for_detached_datafile( md_filter_context_t *mfx, const char *inname );
 
 /*-- comment.c --*/
@@ -310,6 +286,8 @@ int write_comment( IOBUF out, const char *s );
 int make_keysig_packet( PKT_signature **ret_sig, PKT_public_cert *pkc,
 			PKT_user_id *uid, PKT_public_cert *subpkc,
 			PKT_secret_cert *skc,
-			int sigclass, int digest_algo );
+			int sigclass, int digest_algo,
+			int (*mksubpkt)(PKT_signature *, void *),
+			void *opaque  );
 
 #endif /*G10_PACKET_H*/
