@@ -32,6 +32,7 @@
 #include "memory.h"
 #include "filter.h"
 #include "options.h"
+#include "main.h"
 
 static int mpi_print_mode = 0;
 static int list_mode = 0;
@@ -64,6 +65,7 @@ static int  parse_compressed( IOBUF inp, int pkttype, unsigned long pktlen,
 static int  parse_encrypted( IOBUF inp, int pkttype, unsigned long pktlen,
 							   PACKET *packet );
 
+
 static unsigned short
 read_16(IOBUF inp)
 {
@@ -84,16 +86,6 @@ read_32(IOBUF inp)
     return a;
 }
 
-static unsigned long
-buffer_to_u32( const byte *buffer )
-{
-    unsigned long a;
-    a =  *buffer << 24;
-    a |= buffer[1] << 16;
-    a |= buffer[2] << 8;
-    a |= buffer[3];
-    return a;
-}
 
 int
 set_packet_list_mode( int mode )
@@ -318,6 +310,7 @@ parse( IOBUF inp, PACKET *pkt, int reqtype, ulong *retpos,
 	break;
       case PKT_RING_TRUST:
 	parse_trust(inp, pkttype, pktlen);
+	rc = 0;
 	break;
       case PKT_PLAINTEXT:
 	rc = parse_plaintext(inp, pkttype, pktlen, pkt );
@@ -502,6 +495,7 @@ static int
 parse_pubkeyenc( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *packet )
 {
     unsigned n;
+    int i, ndata;
     PKT_pubkey_enc *k;
 
     k = packet->pkt.pubkey_enc = m_alloc(sizeof *packet->pkt.pubkey_enc );
@@ -520,31 +514,20 @@ parse_pubkeyenc( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *packet )
     if( list_mode )
 	printf(":pubkey enc packet: version %d, algo %d, keyid %08lX%08lX\n",
 	  k->version, k->pubkey_algo, (ulong)k->keyid[0], (ulong)k->keyid[1]);
-    if( is_ELGAMAL(k->pubkey_algo) ) {
-	n = pktlen;
-	k->d.elg.a = mpi_read(inp, &n, 0); pktlen -=n;
-	n = pktlen;
-	k->d.elg.b = mpi_read(inp, &n, 0 ); pktlen -=n;
-	if( list_mode ) {
-	    printf("\telg a: ");
-	    mpi_print(stdout, k->d.elg.a, mpi_print_mode );
-	    printf("\n\telg b: ");
-	    mpi_print(stdout, k->d.elg.b, mpi_print_mode );
-	    putchar('\n');
-	}
-    }
-    else if( is_RSA(k->pubkey_algo) ) {
-	n = pktlen;
-	k->d.rsa.rsa_integer = mpi_read(inp, &n, 0 ); pktlen -=n;
-	if( list_mode ) {
-	    printf("\trsa integer: ");
-	    mpi_print(stdout, k->d.rsa.rsa_integer, mpi_print_mode );
-	    putchar('\n');
-	}
-    }
-    else if( list_mode )
-	printf("\tunknown algorithm %d\n", k->pubkey_algo );
 
+    ndata = pubkey_get_nenc(k->pubkey_algo);
+    if( !ndata && list_mode )
+	printf("\tunsupported algorithm %d\n", k->pubkey_algo );
+
+    for( i=0; i < ndata; i++ ) {
+	n = pktlen;
+	k->data[i] = mpi_read(inp, &n, 0); pktlen -=n;
+	if( list_mode ) {
+	    printf("\tdata: ");
+	    mpi_print(stdout, k->data[i], mpi_print_mode );
+	    putchar('\n');
+	}
+    }
 
   leave:
     skip_rest(inp, pktlen);
@@ -615,6 +598,7 @@ parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
 	   : type == SIGSUBPKT_POLICY	   ? "policy URL"
 	   : type == SIGSUBPKT_KEY_FLAGS   ? "key flags"
 	   : type == SIGSUBPKT_SIGNERS_UID ? "signer's user id"
+	   : type == SIGSUBPKT_PRIV_ADD_SIG? "signs additional user id"
 			      : "?");
 	}
 	else if( type == reqtype )
@@ -629,6 +613,8 @@ parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
 	*ret_n = n;
     switch( type ) {
       case SIGSUBPKT_SIG_CREATED:
+      case SIGSUBPKT_SIG_EXPIRE:
+      case SIGSUBPKT_KEY_EXPIRE:
 	if( n < 4 )
 	    break;
 	return buffer;
@@ -636,6 +622,13 @@ parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
 	if( n < 8 )
 	    break;
 	return buffer;
+      case SIGSUBPKT_PRIV_ADD_SIG:
+	/* because we use private data, we check the GNUPG marker */
+	if( n < 24 )
+	    break;
+	if( buffer[0] != 'G' || buffer[1] != 'P' || buffer[2] != 'G' )
+	    return NULL;
+	return buffer+3;
       default: BUG(); /* not yet needed */
     }
     log_error("subpacket of type %d too short\n", type);
@@ -655,6 +648,7 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
     unsigned n;
     int is_v4=0;
     int rc=0;
+    int i, ndata;
 
     if( pktlen < 16 ) {
 	log_error("packet(%d) too short\n", pkttype);
@@ -755,44 +749,22 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
 	    parse_sig_subpkt( sig->unhashed_data,SIGSUBPKT_LIST_UNHASHED, NULL);
 	}
     }
-    if( is_ELGAMAL(sig->pubkey_algo) ) {
-	n = pktlen;
-	sig->d.elg.a = mpi_read(inp, &n, 0 ); pktlen -=n;
-	n = pktlen;
-	sig->d.elg.b = mpi_read(inp, &n, 0 ); pktlen -=n;
-	if( list_mode ) {
-	    printf("\telg a: ");
-	    mpi_print(stdout, sig->d.elg.a, mpi_print_mode );
-	    printf("\n\telg b: ");
-	    mpi_print(stdout, sig->d.elg.b, mpi_print_mode );
-	    putchar('\n');
-	}
-    }
-    else if( sig->pubkey_algo == PUBKEY_ALGO_DSA ) {
-	n = pktlen;
-	sig->d.dsa.r = mpi_read(inp, &n, 0 ); pktlen -=n;
-	n = pktlen;
-	sig->d.dsa.s = mpi_read(inp, &n, 0 ); pktlen -=n;
-	if( list_mode ) {
-	    printf("\tdsa r: ");
-	    mpi_print(stdout, sig->d.elg.a, mpi_print_mode );
-	    printf("\n\tdsa s: ");
-	    mpi_print(stdout, sig->d.elg.b, mpi_print_mode );
-	    putchar('\n');
-	}
-    }
-    else if( is_RSA(sig->pubkey_algo) ) {
-	n = pktlen;
-	sig->d.rsa.rsa_integer = mpi_read(inp, &n, 0 ); pktlen -=n;
-	if( list_mode ) {
-	    printf("\trsa integer: ");
-	    mpi_print(stdout, sig->d.rsa.rsa_integer, mpi_print_mode );
-	    putchar('\n');
-	}
-    }
-    else if( list_mode )
+
+    ndata = pubkey_get_nsig(sig->pubkey_algo);
+    if( !ndata && list_mode )
 	printf("\tunknown algorithm %d\n", sig->pubkey_algo );
 
+    for( i=0; i < ndata; i++ ) {
+	n = pktlen;
+	sig->data[i] = mpi_read(inp, &n, 0 );
+	pktlen -=n;
+	if( list_mode ) {
+	    printf("\tdata: ");
+	    mpi_print(stdout, sig->data[i], mpi_print_mode );
+	    putchar('\n');
+	}
+    }
+
 
   leave:
     skip_rest(inp, pktlen);
@@ -924,17 +896,17 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
 	    putchar('\n');
 	}
 	if( pkttype == PKT_PUBLIC_CERT || pkttype == PKT_PUBKEY_SUBCERT ) {
-	    pkt->pkt.public_cert->d.elg.p = elg_p;
-	    pkt->pkt.public_cert->d.elg.g = elg_g;
-	    pkt->pkt.public_cert->d.elg.y = elg_y;
+	    pkt->pkt.public_cert->pkey[0] = elg_p;
+	    pkt->pkt.public_cert->pkey[1] = elg_g;
+	    pkt->pkt.public_cert->pkey[2] = elg_y;
 	}
 	else {
 	    PKT_secret_cert *cert = pkt->pkt.secret_cert;
 	    byte temp[8];
 
-	    pkt->pkt.secret_cert->d.elg.p = elg_p;
-	    pkt->pkt.secret_cert->d.elg.g = elg_g;
-	    pkt->pkt.secret_cert->d.elg.y = elg_y;
+	    pkt->pkt.secret_cert->skey[0] = elg_p;
+	    pkt->pkt.secret_cert->skey[1] = elg_g;
+	    pkt->pkt.secret_cert->skey[2] = elg_y;
 	    cert->protect.algo = iobuf_get_noeof(inp); pktlen--;
 	    if( cert->protect.algo ) {
 		cert->is_protected = 1;
@@ -994,15 +966,15 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
 
 		}
 		else {
-		    if( list_mode )
-			printf(  "\tprotect algo: %d\n",
-						cert->protect.algo);
 		    /* old version, we don't have a S2K, so we fake one */
 		    cert->protect.s2k.mode = 0;
 		    /* We need this kludge to cope with old GNUPG versions */
 		    cert->protect.s2k.hash_algo =
 			 cert->protect.algo == CIPHER_ALGO_BLOWFISH160?
 				      DIGEST_ALGO_RMD160 : DIGEST_ALGO_MD5;
+		    if( list_mode )
+			printf(  "\tprotect algo: %d  (hash algo: %d)\n",
+			     cert->protect.algo, cert->protect.s2k.hash_algo );
 		}
 		if( pktlen < 8 ) {
 		    rc = G10ERR_INVALID_PACKET;
@@ -1024,20 +996,15 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
 	     * If the user is so careless, not to protect his secret key,
 	     * we can assume, that he operates an open system :=(.
 	     * So we put the key into secure memory when we unprotect it. */
-	    n = pktlen; cert->d.elg.x = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    n = pktlen; cert->skey[3] = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    if( cert->is_protected )
+		mpi_set_protect_flag(cert->skey[3]);
 
 	    cert->csum = read_16(inp); pktlen -= 2;
 	    if( list_mode ) {
-	    printf("\telg x: ");
-	    mpi_print(stdout, cert->d.elg.x, mpi_print_mode  );
-	    putchar('\n');
 		printf("\t[secret value x is not shown]\n"
 		       "\tchecksum: %04hx\n", cert->csum);
 	    }
-	  /*log_mpidump("elg p=", cert->d.elg.p );
-	    log_mpidump("elg g=", cert->d.elg.g );
-	    log_mpidump("elg y=", cert->d.elg.y );
-	    log_mpidump("elg x=", cert->d.elg.x ); */
 	}
     }
     else if( algorithm == PUBKEY_ALGO_DSA ) {
@@ -1058,19 +1025,19 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
 	    putchar('\n');
 	}
 	if( pkttype == PKT_PUBLIC_CERT || pkttype == PKT_PUBKEY_SUBCERT ) {
-	    pkt->pkt.public_cert->d.dsa.p = dsa_p;
-	    pkt->pkt.public_cert->d.dsa.q = dsa_q;
-	    pkt->pkt.public_cert->d.dsa.g = dsa_g;
-	    pkt->pkt.public_cert->d.dsa.y = dsa_y;
+	    pkt->pkt.public_cert->pkey[0] = dsa_p;
+	    pkt->pkt.public_cert->pkey[1] = dsa_q;
+	    pkt->pkt.public_cert->pkey[2] = dsa_g;
+	    pkt->pkt.public_cert->pkey[3] = dsa_y;
 	}
 	else {
 	    PKT_secret_cert *cert = pkt->pkt.secret_cert;
 	    byte temp[8];
 
-	    pkt->pkt.secret_cert->d.dsa.p = dsa_p;
-	    pkt->pkt.secret_cert->d.dsa.q = dsa_q;
-	    pkt->pkt.secret_cert->d.dsa.g = dsa_g;
-	    pkt->pkt.secret_cert->d.dsa.y = dsa_y;
+	    pkt->pkt.secret_cert->skey[0] = dsa_p;
+	    pkt->pkt.secret_cert->skey[1] = dsa_q;
+	    pkt->pkt.secret_cert->skey[2] = dsa_g;
+	    pkt->pkt.secret_cert->skey[3] = dsa_y;
 	    cert->protect.algo = iobuf_get_noeof(inp); pktlen--;
 	    if( cert->protect.algo ) {
 		cert->is_protected = 1;
@@ -1156,18 +1123,15 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
 	     * If the user is so careless, not to protect his secret key,
 	     * we can assume, that he operates an open system :=(.
 	     * So we put the key into secure memory when we unprotect it. */
-	    n = pktlen; cert->d.dsa.x = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    n = pktlen; cert->skey[4] = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    if( cert->is_protected )
+		mpi_set_protect_flag(cert->skey[4]);
 
 	    cert->csum = read_16(inp); pktlen -= 2;
 	    if( list_mode ) {
 		printf("\t[secret value x is not shown]\n"
 		       "\tchecksum: %04hx\n", cert->csum);
 	    }
-	  /*log_mpidump("dsa p=", cert->d.dsa.p );
-	    log_mpidump("dsa q=", cert->d.dsa.q );
-	    log_mpidump("dsa g=", cert->d.dsa.g );
-	    log_mpidump("dsa y=", cert->d.dsa.y );
-	    log_mpidump("dsa x=", cert->d.dsa.x ); */
 	}
     }
     else if( is_RSA(algorithm) ) {
@@ -1183,15 +1147,15 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
 	    putchar('\n');
 	}
 	if( pkttype == PKT_PUBLIC_CERT || pkttype == PKT_PUBKEY_SUBCERT ) {
-	    pkt->pkt.public_cert->d.rsa.n = rsa_pub_mod;
-	    pkt->pkt.public_cert->d.rsa.e = rsa_pub_exp;
+	    pkt->pkt.public_cert->pkey[0] = rsa_pub_mod;
+	    pkt->pkt.public_cert->pkey[1] = rsa_pub_exp;
 	}
 	else {
 	    PKT_secret_cert *cert = pkt->pkt.secret_cert;
 	    byte temp[8];
 
-	    pkt->pkt.secret_cert->d.rsa.n = rsa_pub_mod;
-	    pkt->pkt.secret_cert->d.rsa.e = rsa_pub_exp;
+	    pkt->pkt.secret_cert->skey[0] = rsa_pub_mod;
+	    pkt->pkt.secret_cert->skey[1] = rsa_pub_exp;
 	    cert->protect.algo = iobuf_get_noeof(inp); pktlen--;
 	    if( list_mode )
 		printf(  "\tprotect algo: %d\n", cert->protect.algo);
@@ -1205,28 +1169,30 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
 			printf(" %02x", temp[i] );
 		    putchar('\n');
 		}
-		if( cert->protect.algo == CIPHER_ALGO_BLOWFISH160 )
-		    memcpy(cert->protect.iv, temp, 8 );
+		memcpy(cert->protect.iv, temp, 8 );
+		/* old version, we don't have a S2K, so we fake one */
+		cert->protect.s2k.mode = 0;
+		cert->protect.s2k.hash_algo = DIGEST_ALGO_MD5;
 	    }
 	    else
 		cert->is_protected = 0;
 	    /* (See comments at the code for elg keys) */
-	    n = pktlen; cert->d.rsa.d = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    n = pktlen; cert->d.rsa.p = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    n = pktlen; cert->d.rsa.q = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    n = pktlen; cert->d.rsa.u = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    n = pktlen; cert->skey[2] = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    n = pktlen; cert->skey[3] = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    n = pktlen; cert->skey[4] = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    n = pktlen; cert->skey[5] = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    if( cert->is_protected ) {
+		mpi_set_protect_flag(cert->skey[2]);
+		mpi_set_protect_flag(cert->skey[3]);
+		mpi_set_protect_flag(cert->skey[4]);
+		mpi_set_protect_flag(cert->skey[5]);
+	    }
 
 	    cert->csum = read_16(inp); pktlen -= 2;
 	    if( list_mode ) {
 		printf("\t[secret values d,p,q,u are not shown]\n"
 		       "\tchecksum: %04hx\n", cert->csum);
 	    }
-	 /* log_mpidump("rsa n=", cert->d.rsa.n );
-	    log_mpidump("rsa e=", cert->d.rsa.e );
-	    log_mpidump("rsa d=", cert->d.rsa.d );
-	    log_mpidump("rsa p=", cert->d.rsa.p );
-	    log_mpidump("rsa q=", cert->d.rsa.q );
-	    log_mpidump("rsa u=", cert->d.rsa.u ); */
 	}
     }
     else if( list_mode )
