@@ -38,22 +38,47 @@
 #include "i18n.h"
 
 
+static int
+do_sign( PKT_secret_cert *skc, PKT_signature *sig,
+	 MD_HANDLE md, int digest_algo )
+{
+    MPI frame;
+    byte *dp;
+    int rc;
+
+    if( !digest_algo )
+	digest_algo = md_get_algo(md);
+
+    dp = md_read( md, digest_algo );
+    sig->digest_algo = digest_algo;
+    sig->digest_start[0] = dp[0];
+    sig->digest_start[1] = dp[1];
+    frame = encode_md_value( skc->pubkey_algo, md,
+			     digest_algo, mpi_get_nbits(skc->skey[0]));
+    rc = pubkey_sign( skc->pubkey_algo, sig->data, frame, skc->skey );
+    mpi_free(frame);
+    if( rc )
+	log_error("pubkey_sign failed: %s\n", g10_errstr(rc) );
+    else {
+	if( opt.verbose ) {
+	    char *ustr = get_user_id_string( sig->keyid );
+	    log_info("%s signature from: %s\n",
+		      pubkey_algo_to_string(skc->pubkey_algo), ustr );
+	    m_free(ustr);
+	}
+    }
+    return rc;
+}
+
+
 
 int
 complete_sig( PKT_signature *sig, PKT_secret_cert *skc, MD_HANDLE md )
 {
     int rc=0;
 
-    if( (rc=check_secret_key( skc )) )
-	;
-    else if( is_ELGAMAL(sig->pubkey_algo) )
-	g10_elg_sign( skc, sig, md, 0 );
-    else if( sig->pubkey_algo == PUBKEY_ALGO_DSA )
-	g10_dsa_sign( skc, sig, md, 0 );
-    else if( is_RSA(sig->pubkey_algo) )
-	g10_rsa_sign( skc, sig, md, 0 );
-    else
-	BUG();
+    if( !(rc=check_secret_key( skc )) )
+	rc = do_sign( skc, sig, md, 0 );
 
     /* fixme: should we check whether the signature is okay?
      * maybe by using an option */
@@ -73,7 +98,22 @@ hash_for(int pubkey_algo )
     return DEFAULT_DIGEST_ALGO;
 }
 
+static int
+only_old_style( SKC_LIST skc_list )
+{
+    SKC_LIST skc_rover = NULL;
+    int old_style = 0;
 
+    /* if there are only old style capable key we use the old sytle */
+    for( skc_rover = skc_list; skc_rover; skc_rover = skc_rover->next ) {
+	PKT_secret_cert *skc = skc_rover->skc;
+	if( skc->pubkey_algo == PUBKEY_ALGO_RSA && skc->version < 4 )
+	    old_style = 1;
+	else
+	    return 0;
+    }
+    return old_style;
+}
 
 /****************
  * Sign the files whose names are in FILENAME.
@@ -106,6 +146,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     SKC_LIST skc_list = NULL;
     SKC_LIST skc_rover = NULL;
     int multifile = 0;
+    int old_style = opt.rfc1991;
+
 
     memset( &afx, 0, sizeof afx);
     memset( &zfx, 0, sizeof zfx);
@@ -126,6 +168,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 
     if( (rc=build_skc_list( locusr, &skc_list, 1, 1 )) )
 	goto leave;
+    if( !old_style )
+	old_style = only_old_style( skc_list );
     if( encrypt ) {
 	if( (rc=build_pkc_list( remusr, &pkc_list, 2 )) )
 	    goto leave;
@@ -179,11 +223,13 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	iobuf_push_filter( out, encrypt_filter, &efx );
     }
 
-    if( opt.compress && !outfile )
+    if( opt.compress && !outfile ) {
+	if( old_style )
+	    zfx.algo = 1;
 	iobuf_push_filter( out, compress_filter, &zfx );
+    }
 
-
-    if( !detached ) {
+    if( !detached && !old_style ) {
 	/* loop over the secret certificates and build headers */
 	for( skc_rover = skc_list; skc_rover; skc_rover = skc_rover->next ) {
 	    PKT_secret_cert *skc;
@@ -254,6 +300,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	     * data, it is not possible to know the used length
 	     * without a double read of the file - to avoid that
 	     * we simple use partial length packets.
+	     * FIXME: We have to do the double read when opt.rfc1991
+	     *	      is active.
 	     */
 	    if( opt.textmode && !outfile )
 		filesize = 0;
@@ -332,27 +380,20 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	}
 	md_final( md );
 
-	if( is_ELGAMAL(sig->pubkey_algo) )
-	    g10_elg_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
-	else if( sig->pubkey_algo == PUBKEY_ALGO_DSA )
-	    g10_dsa_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
-	else if( is_RSA(sig->pubkey_algo) )
-	    g10_rsa_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
-	else
-	    BUG();
-
+	rc = do_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
 	md_close( md );
 
-	/* and write it */
-	init_packet(&pkt);
-	pkt.pkttype = PKT_SIGNATURE;
-	pkt.pkt.signature = sig;
-	rc = build_packet( out, &pkt );
-	free_packet( &pkt );
-	if( rc ) {
-	    log_error("build signature packet failed: %s\n", g10_errstr(rc) );
-	    goto leave;
+	if( !rc ) { /* and write it */
+	    init_packet(&pkt);
+	    pkt.pkttype = PKT_SIGNATURE;
+	    pkt.pkt.signature = sig;
+	    rc = build_packet( out, &pkt );
+	    free_packet( &pkt );
+	    if( rc )
+		log_error("build signature packet failed: %s\n", g10_errstr(rc) );
 	}
+	if( rc )
+	    goto leave;
     }
 
 
@@ -419,6 +460,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     int rc = 0;
     SKC_LIST skc_list = NULL;
     SKC_LIST skc_rover = NULL;
+    int old_style = opt.rfc1991;
 
     memset( &afx, 0, sizeof afx);
     memset( &tfx, 0, sizeof tfx);
@@ -426,6 +468,8 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 
     if( (rc=build_skc_list( locusr, &skc_list, 1, 1 )) )
 	goto leave;
+    if( !old_style )
+	old_style = only_old_style( skc_list );
 
     /* prepare iobufs */
     if( !(inp = iobuf_open(fname)) ) {
@@ -449,9 +493,10 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	goto leave;
     }
 
-    /* FIXME: This stuff is not correct if mutliplehash algos are used*/
+    /* FIXME: This stuff is not correct if multiple hash algos are used*/
     iobuf_writestr(out, "-----BEGIN PGP SIGNED MESSAGE-----\n" );
-    if( (opt.def_digest_algo?opt.def_digest_algo:DEFAULT_DIGEST_ALGO)
+    if( old_style
+	|| (opt.def_digest_algo?opt.def_digest_algo:DEFAULT_DIGEST_ALGO)
 			      == DIGEST_ALGO_MD5 )
 	iobuf_writestr(out, "\n" );
     else {
@@ -535,27 +580,20 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	}
 	md_final( md );
 
-	if( is_ELGAMAL(sig->pubkey_algo) )
-	    g10_elg_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
-	else if( sig->pubkey_algo == PUBKEY_ALGO_DSA )
-	    g10_dsa_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
-	else if( is_RSA(sig->pubkey_algo) )
-	    g10_rsa_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
-	else
-	    BUG();
-
+	rc = do_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
 	md_close( md );
 
-	/* and write it */
-	init_packet(&pkt);
-	pkt.pkttype = PKT_SIGNATURE;
-	pkt.pkt.signature = sig;
-	rc = build_packet( out, &pkt );
-	free_packet( &pkt );
-	if( rc ) {
-	    log_error("build signature packet failed: %s\n", g10_errstr(rc) );
-	    goto leave;
+	if( !rc ) { /* and write it */
+	    init_packet(&pkt);
+	    pkt.pkttype = PKT_SIGNATURE;
+	    pkt.pkt.signature = sig;
+	    rc = build_packet( out, &pkt );
+	    free_packet( &pkt );
+	    if( rc )
+		log_error("build signature packet failed: %s\n", g10_errstr(rc) );
 	}
+	if( rc )
+	    goto leave;
     }
 
 
