@@ -1,19 +1,14 @@
 /* cipher.h
- *	Copyright (c) 1997 by Werner Koch (dd9jn)
+ *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * ATTENTION: This code should not be exported from the United States
- * nor should it be used their without a license agreement with PKP.
- * The RSA alorithm is protected by U.S. Patent #4,405,829 which
- * expires on September 20, 2000!
+ * This file is part of GNUPG.
  *
- * This file is part of G10.
- *
- * G10 is free software; you can redistribute it and/or modify
+ * GNUPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * G10 is distributed in the hope that it will be useful,
+ * GNUPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -25,22 +20,16 @@
 #ifndef G10_CIPHER_H
 #define G10_CIPHER_H
 
-#define DBG_CIPHER cipher_debug_mode
+#define DBG_CIPHER g10c_debug_mode
 
 #include "mpi.h"
-#include "../cipher/md.h"
-#ifdef HAVE_RSA_CIPHER
-  #include "../cipher/rsa.h"
-#endif
-#include "../cipher/elgamal.h"
-#include "../cipher/dsa.h"
 #include "../cipher/random.h"
 
 
 #define CIPHER_ALGO_NONE	 0
 #define CIPHER_ALGO_IDEA	 1
 #define CIPHER_ALGO_3DES	 2
-#define CIPHER_ALGO_CAST	 3
+#define CIPHER_ALGO_CAST5	 3
 #define CIPHER_ALGO_BLOWFISH	 4  /* blowfish 128 bit key */
 #define CIPHER_ALGO_SAFER_SK128  5
 #define CIPHER_ALGO_DES_SK	 6
@@ -57,9 +46,7 @@
 #define DIGEST_ALGO_MD5       1
 #define DIGEST_ALGO_SHA1      2
 #define DIGEST_ALGO_RMD160    3
-#ifdef WITH_TIGER_HASH
 #define DIGEST_ALGO_TIGER     6
-#endif
 
 #define is_RSA(a)     ((a)==PUBKEY_ALGO_RSA || (a)==PUBKEY_ALGO_RSA_E \
 		       || (a)==PUBKEY_ALGO_RSA_S )
@@ -84,7 +71,55 @@ struct cipher_handle_s { char does_not_matter[1]; };
 #define CIPHER_MODE_DUMMY     5  /* used with algo DUMMY for no encryption */
 
 
-int cipher_debug_mode;
+#define MD_BUFFER_SIZE 512
+
+typedef struct {
+    byte buffer[MD_BUFFER_SIZE];
+    int  bufcount;
+    int  secure;
+    FILE  *debug;
+    int guard1;
+    struct md_digest_list_s *list;
+    int guard2;
+} *MD_HANDLE;
+
+
+#ifndef DEFINES_MD_HANDLE /* not really the handle but the algorithm list */
+struct md_digest_list_s { char does_not_matter[1]; };
+#endif
+
+
+
+int g10c_debug_mode;
+int g10_opt_verbose;
+
+/*-- dynload.c --*/
+void register_cipher_extension( const char *fname );
+
+/*-- md.c --*/
+int string_to_digest_algo( const char *string );
+const char * digest_algo_to_string( int algo );
+int check_digest_algo( int algo );
+MD_HANDLE md_open( int algo, int secure );
+void md_enable( MD_HANDLE hd, int algo );
+MD_HANDLE md_copy( MD_HANDLE a );
+void md_close(MD_HANDLE a);
+void md_write( MD_HANDLE a, byte *inbuf, size_t inlen);
+void md_final(MD_HANDLE a);
+byte *md_read( MD_HANDLE a, int algo );
+int md_get_algo( MD_HANDLE a );
+int md_digest_length( int algo );
+const byte *md_asn_oid( int algo, size_t *asnlen, size_t *mdlen );
+void md_start_debug( MD_HANDLE a, const char *suffix );
+void md_stop_debug( MD_HANDLE a );
+#define md_is_secure(a) ((a)->secure)
+#define md_putc(h,c)					    \
+	    do {					    \
+		if( (h)->bufcount == MD_BUFFER_SIZE )	    \
+		    md_write( (h), NULL, 0 );		    \
+		(h)->buffer[(h)->bufcount++] = (c) & 0xff;  \
+	    } while(0)
+
 
 /*-- cipher.c --*/
 int string_to_cipher_algo( const char *string );
@@ -99,15 +134,28 @@ void cipher_encrypt( CIPHER_HANDLE c, byte *out, byte *in, unsigned nbytes );
 void cipher_decrypt( CIPHER_HANDLE c, byte *out, byte *in, unsigned nbytes );
 void cipher_sync( CIPHER_HANDLE c );
 
+/*-- pubkey.c --*/
+#define PUBKEY_MAX_NPKEY  4
+#define PUBKEY_MAX_NSKEY  6
+#define PUBKEY_MAX_NSIG   2
+#define PUBKEY_MAX_NENC   2
 
-/*-- misc.c --*/
 int string_to_pubkey_algo( const char *string );
-int string_to_digest_algo( const char *string );
 const char * pubkey_algo_to_string( int algo );
-const char * digest_algo_to_string( int algo );
 int check_pubkey_algo( int algo );
 int check_pubkey_algo2( int algo, unsigned usage );
-int check_digest_algo( int algo );
+int pubkey_get_npkey( int algo );
+int pubkey_get_nskey( int algo );
+int pubkey_get_nsig( int algo );
+int pubkey_get_nenc( int algo );
+unsigned pubkey_nbits( int algo, MPI *pkey );
+int pubkey_generate( int algo, unsigned nbits, MPI *skey, MPI **retfactors );
+int pubkey_check_secret_key( int algo, MPI *skey );
+int pubkey_encrypt( int algo, MPI *resarr, MPI data, MPI *pkey );
+int pubkey_decrypt( int algo, MPI *result, MPI *data, MPI *skey );
+int pubkey_sign( int algo, MPI *resarr, MPI hash, MPI *skey );
+int pubkey_verify( int algo, MPI hash, MPI *data, MPI *pkey,
+		      int (*cmp)(void *, MPI), void *opaque );
 
 /*-- smallprime.c --*/
 extern ushort small_prime_numbers[];
